Program PTune;
{Used for channel tuning in the Philo app}

{$mode objfpc}

Uses
 classes, sysutils, inifiles, httpsend, synacode;

Procedure WriteStringList(const msg: String; const sl: TStringList);
 Var
  i: Integer;
 Begin
  writeln(msg);
  for i:=0 to sl.Count-1 do
   writeln('   ',sl.Strings[i]);
 End;

Function TimeStamp: String;
 Begin
  TimeStamp:=FormatDateTime('ddd m/d/yyyy h:nn.ssam/pm', Now);
 End;

Function GetFileSize(const fName: String): Int64;
 Var
  sr: TSearchRec;
 Begin
  if FindFirst(fName,faAnyFile and faDirectory,sr)=0 then
   GetFileSize:=sr.Size
    Else
     GetFileSize:=0;
 End;

{Returns the app ID of the Philo app}
Function GetPhiloAppID(const rokuIP: String; var appID: String): Boolean;
 Var
  httpClient: THTTPSend;
  data: TStringList;
  i,p: Word;
 Begin
  result:=false;
  {Init string list}
  data:=TStringList.Create;
  {Init HTTP Library}
  httpClient:=THTTPSend.Create;
  try
   data.Clear;
   result:=httpClient.HTTPMethod('GET','http://'+rokuIP+':8060/query/apps');
   data.LoadFromStream(httpClient.Document);
   i:=0;
   while i<data.Count do
    Begin
     if pos('>Philo<',data[i])>1 then
      Begin
       p:=pos('<app id="',data[i]);
       inc(p,9);
       while (p<length(data[i])) and (data[i][p]<>'"') do
        Begin
         appID:=appID+data[i][p];
         inc(p);
        End;
       //writeln(Timestamp+'  App ID=',appID);
      End;
     inc(i);
    End;
  finally
   data.Free;
   httpClient.Free;
  end;
 End;

{Returns the app ID of the current app, or "Roku" if at the main menu}
Function GetActiveApp(const rokuIP: String; var activeAppID: String): Boolean;
 Var
  httpClient: THTTPSend;
  data: TStringList;
  i,p: Word;
 Begin
  result:=false;
  activeAppID:='!@#$';
  {Init string list}
  data:=TStringList.Create;
  {Init HTTP Library}
  httpClient:=THTTPSend.Create;
  try
   data.Clear;
   result:=httpClient.HTTPMethod('GET','http://'+rokuIP+':8060/query/active-app');
   data.LoadFromStream(httpClient.Document);
   i:=0;
   while i<data.Count do
    Begin
     p:=pos('<app',data[i]);
     if p>0 then
      Begin
       if pos('<app>',data[i])>0 then
        Begin
         activeAppID:='';
         p:=p+5;
         while (p<length(data[i])) and (data[i][p]<>'<') do
          Begin
           activeAppID:=activeAppID+data[i][p];
           inc(p);
          End;
         End
         Else
          Begin
           activeAppID:='';
           p:=pos('<app id="',data[i]);
           p:=p+9;
           while (p<length(data[i])) and (data[i][p]<>'"') do
            Begin
             activeAppID:=activeAppID+data[i][p];
             inc(p);
            End;
          End;
       writeln(TimeStamp+'  Roku active app ID is ',activeAppID);
       //writeln(buffer, TimeStamp'  Roku active app ID is ',activeAppID);
      End;
     inc(i);
    End;
  finally
   data.Free;
   httpClient.Free;
  end;
 End;

{Launch an app on the Roku}
Function Launch(const rokuIP, appID: String): Boolean;
 Var
  httpClient: THTTPSend;
  data: TStringList;
 Begin
  result:=false;
  {Init string list}
  data:=TStringList.Create;
  {Init HTTP Library}
  httpClient:=THTTPSend.Create;
  try
   data.Clear;
   result:=httpClient.HTTPMethod('POST','http://'+rokuIP+':8060/launch/'+appID);
   data.LoadFromStream(httpClient.Document);
  finally
   data.Free;
   httpClient.Free;
  end;
 End;

{Pushes a "button" on the Roku remote}
Function Keypress(const rokuIP, key: String): Boolean;
 Var
  httpClient: THTTPSend;
  data: TStringList;
 Begin
  result:=false;
  {Init string list}
  data:=TStringList.Create;
  {Init HTTP Library}
  httpClient:=THTTPSend.Create;
  try
   data.Clear;
   result:=httpClient.HTTPMethod('POST','http://'+rokuIP+':8060/keypress/'+key);
   data.LoadFromStream(httpClient.Document);
  finally
   data.Free;
   httpClient.Free;
  end;
 End;

Procedure LoadConfig(const tuningDevice: String; var appID: String;
                     var homeWaitTime, launchWaitTime, menuDrawWaitTime: Word;
                     var menuScrollWaitTime, playWaitTime: Word;
                     var sagexUser, sagexPass: String);
 Var
  ptuneIni: TIniFile;
  configFile: String;
  temp: Integer;
 Begin
  configFile:=ExtractFilePath(ParamStr(0))+'ptune.ini';
  ptuneIni:=TIniFile.Create(configFile);
  appID:=ptuneIni.ReadString(tuningDevice,'AppID','');
  homeWaitTime:=ptuneIni.ReadInteger(tuningDevice,'HomeWaitTime',2000);
  launchWaitTime:=ptuneIni.ReadInteger(tuningDevice,'LaunchWaitTime',5000);
  menuDrawWaitTime:=ptuneIni.ReadInteger(tuningDevice,'MenuDrawWaitTime',2000);
  menuScrollWaitTime:=ptuneIni.ReadInteger(tuningDevice,'MenuScrollWaitTime',150);
  playWaitTime:=ptuneIni.ReadInteger(tuningDevice,'PlayWaitTime',0);
  sagexUser:=ptuneIni.ReadString('SageX','Username','sage');
  sagexPass:=ptuneIni.ReadString('SageX','Password','frey');
  ptuneIni.Free;
 End;

Procedure SaveConfig(const tuningDevice: String; channel: Word;
                     const appID: String;
                     homeWaitTime, launchWaitTime, menuDrawWaitTime: Word;
                     menuScrollWaitTime, playWaitTime: Word;
                     const sagexUser, sagexPass: String);
 Var
  ptuneIni: TIniFile;
  configFile: String;
 Begin
  configFile:=ExtractFilePath(ParamStr(0))+'ptune.ini';
  ptuneIni:=TIniFile.Create(configFile);
  if channel>0 then
   Begin
    ptuneIni.WriteString(tuningDevice,'AppID',appID);
    ptuneIni.WriteInteger(tuningDevice,'HomeWaitTime',homeWaitTime);
    ptuneIni.WriteInteger(tuningDevice,'LaunchWaitTime',launchWaitTime);
    ptuneIni.WriteInteger(tuningDevice,'MenuDrawWaitTime',menuDrawWaitTime);
    ptuneIni.WriteInteger(tuningDevice,'MenuScrollWaitTime',menuScrollWaitTime);
    ptuneIni.WriteInteger(tuningDevice,'PlayWaitTime',playWaitTime);
   End
    Else
     Begin
      ptuneIni.WriteString('SageX','Username',sagexUser);
      ptuneIni.WriteString('SageX','Password',sagexPass);
     End;
  ptuneIni.Free;
 End;

Procedure Init(var tuningDevice: String; var channel: Word;
               var rokuIP: String; var sageIP: String; var appID: String;
               var homeWaitTime, launchWaitTime, menuDrawWaitTime: Word;
               var menuScrollWaitTime, playWaitTime: Word;
               var sagexUser, sagexPass: String);
 Var
  chStr: String;
  code: Word;
 Begin
  tuningDevice:='';
  channel:=0;
  rokuIP:='';
  sageIP:='';
  appID:='';
  if (ParamCount=3) or (ParamCount=4) then
   Begin
    tuningDevice:=ParamStr(1);
    chStr:=Paramstr(2);
    val(chStr,channel,code);
    if code<>0 then channel:=0;
    rokuIP:=ParamStr(3);
    if ParamCount=4 then
     sageIP:=ParamStr(4);
   End;
{
  tuningDevice:='Roku1';
  channel:=0;
  rokuIP:='192.168.1.240';
  sageIP:='192.168.1.128';}

  if (tuningDevice<>'') and (rokuIP<>'') then
   Begin
    LoadConfig(tuningDevice,appID,homeWaitTime,launchWaitTime,
               menuDrawWaitTime, menuScrollWaitTime,playWaitTime,
               sagexUser, sagexPass);
    if appID='' then
     GetPhiloAppID(rokuIP,appID);
   End;
 End;

Procedure Tune(const rokuIP, pAppID, tuningDevice: String; channel: Word;
               homeWaitTime, launchWaitTime, menuDrawWaitTime,
               menuScrollWaitTime, playWaitTime: Word;
               var buffer: TextFile; logEnabled: Boolean);
 Var
  activeAppID: String;
  i: Word;
 Begin
  if GetActiveApp(rokuIP, activeAppID)=true then
   Begin
    {If not at the main menu, send the home command}
    if activeAppID<>'Roku' then
     Begin
      writeln(TimeStamp+'  Not at main menu, sending Home command on '+tuningDevice);
      if logEnabled then writeln(buffer,TimeStamp+'  Not at main menu, sending Home command on '+tuningDevice);
      KeyPress(rokuIP, 'Home');
      sleep(homeWaitTime); {Default 2000 / which is 2 seconds}
     End;
    {Launch Philo App}
    writeln(TimeStamp+'  Launching Philo app on '+tuningDevice);
    if logEnabled then writeln(buffer,TimeStamp+'  Launching Philo app on '+tuningDevice);
    Launch(rokuIP, pAppID);
    {Wait for app to launch. Default wait time 5000 which is 5 seconds}
    sleep(launchWaitTime);
    writeln(TimeStamp+'  Tuning channel '+IntToStr(channel)+' on '+tuningDevice);
    if logEnabled then writeln(buffer,TimeStamp+'  Tuning channel '+IntToStr(channel)+' on '+tuningDevice);

    {Open the channel menu}
    KeyPress(rokuIP, 'Up');
    KeyPress(rokuIP, 'Right');
    KeyPress(rokuIP, 'Select');
    {Wait for the menu to draw}
    sleep(menuDrawWaitTime); {Default 2000, which is 2 seconds}

    i:=1;
    //writeln('channel=',channel);
    while i<channel do
     Begin
      KeyPress(rokuIP, 'Down');
      sleep(menuScrollWaitTime);
      inc(i);
     End;
    sleep(playWaitTime); {Default 0, set the value in ptune.ini to a higher}
    KeyPress(rokuIP, 'Select'); {value when troubleshooting channel lineups}
   End
    Else
     Begin
      writeln(TimeStamp+'  Could not connect to the Roku.  Check to make sure the IP address is valid.');
      if logEnabled then writeln(buffer,TimeStamp+'  Could not connect to the Roku.  Check to make sure the IP address is valid.');
     End;
 End;

{This procedure locates the capture device input associated with}
 {"tuningDevice" then checks to see if there is an active recording.}
 {SageX is required in order for this to work}
Function SageIsRecording(const tuningDevice, sageIP, sagexUser,
                         sagexPass: String; var buffer: TextFile;
                         logEnabled: Boolean): Boolean;
 Var
  httpClient: THTTPSend;
  data: TStringList;
  captureDeviceInputs: TStringList;
  myCaptureDeviceInput: String;
  i,j,p: word;
  temp: String;
 Begin
  result:=false;
  {Init string lists}
  data:=TStringList.Create;
  captureDeviceInputs:=TStringList.Create;
  {Init HTTP Library}
  httpClient:=THTTPSend.Create;
  try
   {Start by getting a list of caputure device inputs}
   data.Clear;
   if httpClient.HTTPMethod('GET','http://'+sagexUser+':'+sagexPass+'@'+sageIP+
                            ':8080/sagex/api?command=GetConfiguredCaptureDeviceInputs') then
    Begin
     data.LoadFromStream(httpClient.Document);
     //writeln('GET http://'+sagexUser+':'+sagexPass+'@'+sageIP+
     //                      ':8080/sagex/api?command=GetConfiguredCaptureDeviceInputs');
     //writestringlist('getconfiguredcapturedeviceinputs',data);
     i:=0;
     while i<data.Count do
      Begin
       temp:=data[i];
       p:=pos('<Item><![CDATA[',temp);
       if p>0 then
        Begin
         temp:=copy(temp,p+15,length(temp)-p-14);
         p:=pos(']]><',temp);
         temp:=copy(temp,1,p-1);
         captureDeviceInputs.Add(temp);
        End;
       inc(i);
      End;
     {Locate the caputure device input that is associated with "tuningDevice"}
     myCaptureDeviceInput:='';
     i:=0;
     while i<captureDeviceInputs.Count do
      Begin
       data.Clear;
       httpClient.HTTPMethod('GET',EncodeURL('http://'+sagexUser+':'+
                             sagexPass+'@'+sageIP+
                             ':8080/sagex/api?command=GetInfraredTunerRemoteName&1='+captureDeviceInputs[i]));
       data.LoadFromStream(httpClient.Document);
       //writestringlist('getinfraredtunerremotename',data);
       temp:='';
       if data.Count>0 then temp:=data[0];
       p:=pos('<Result><![CDATA[',temp);
       if p>0 then
        Begin
         temp:=copy(temp,p+17,length(temp)-p-16);
         p:=pos(']]><',temp);
         temp:=copy(temp,1,p-1);
         if CompareText(temp,tuningDevice)=0 then {Case insensitive compare}
          myCaptureDeviceInput:=captureDeviceInputs[i];
        End;
       inc(i);
      End;
     if myCaptureDeviceInput<>'' then
      Begin
       data.Clear;
       httpClient.HTTPMethod('GET',EncodeURL('http://'+sagexUser+':'+
                             sagexPass+'@'+sageIP+
                             ':8080/sagex/api?command=GetCaptureDeviceCurrentRecordFile&1='+
                             myCaptureDeviceInput));
                             data.LoadFromStream(httpClient.Document);
       if data.Count>2 then
        Begin
         result:=true;
         writeln(TimeStamp+'  The capture device input ', myCaptureDeviceInput,' is recording from ',tuningDevice,'.  No action needed.');
         if logEnabled then writeln(buffer,TimeStamp+'  The capture device input ', myCaptureDeviceInput,' is recording from ',tuningDevice,'.  No action needed.');
        End
         Else
          Begin
           writeln(TimeStamp+'  The capture device input ', myCaptureDeviceInput,' is not currently recording from ',tuningDevice,'.  Sending "home" command.');
           if logEnabled then writeln(buffer,TimeStamp+'  The capture device input ', myCaptureDeviceInput,' is not currently recording from ',tuningDevice,'.  Sending "home" command.');
          End;
      End
       Else
        Begin
         writeln(TimeStamp+'  Could not locate a capture device input for the "',tuningDevice,'" tuning device.');
         if logEnabled then writeln(buffer,TimeStamp+'  Could not locate a capture device input for the "',tuningDevice,'" tuning device.');
        End;
    End
     Else {If we received no response from the GetConfiguredCaptureDeviceInputs request}
      Begin
       writeln(TimeStamp+'  Could not connect to SageTV.  Check to make sure the IP address is valid and that SageX is installed.');
       if logEnabled then writeln(buffer,TimeStamp+'  Could not connect to SageTV.  Check to make sure the IP address is valid and that SageX is installed.');
      End;
  finally
   captureDeviceInputs.Free;
   data.Free;
   httpClient.Free;
  end;
 End;

Procedure Main;
 Var
  tuningDevice, rokuIP, sageIP: String;
  channel: Word;
  appID: String;
  homeWaitTime, launchWaitTime, menuDrawWaitTime, menuScrollWaitTime: Word;
  playWaitTime: Word;
  sagexUser, sagexPass: String;
  buffer: TextFile;
  logFile: String;
  logEnabled: Boolean;
 Begin
  {Get logging ready to go}
  logEnabled:=false;
  logFile:=ExtractFilePath(ParamStr(0))+'ptune.log';
  assignfile(buffer,logFile);
  {$I-}
  if (not fileexists(logFile)) or (GetFileSize(logFile)>1048576) then
   rewrite(buffer)
    else
     append(buffer);
  if IOResult=0 then logEnabled:=true;
  {$I+}
  {Initialize variables}
  Init(tuningDevice, channel, rokuIP, sageIP, appID, homeWaitTime,
       launchWaitTime, menuDrawWaitTime, menuScrollWaitTime, playWaitTime,
       sagexUser, sagexPass);
  if (rokuIP='') or (tuningDevice='') or ((channel=0) and (sageIP='')) then
   Begin
    writeln('Usage:   ptune TuningDevice Channel RokuIPAddress');
    writeln('Example: ptune Roku1 3 192.168.1.240');
    writeln('           Tunes a channel');
    writeln('Usage:   ptune TuningDevice 0 RokuIPAddress SageTVIPAddress');
    writeln('Example: ptune Roku1 0 192.168.1.240 192.168.1.128');
    writeln('           Meant to be run as a scheduled task that runs repatedly');
    writeln('           Checks to see if the SageTV is using the HDPVR that''s assocated with');
    writeln('            TuningDevice.  Sends the "home" command when this HDPVR is inactive.');
    writeln('           Keeps the Roku from wasting bandwidth when not in use');
    if (logEnabled) and (ParamCount>0) then writeln(buffer,TimeStamp+'  Invalid command line parameter(s) used.');
   End
    Else if (appID='') and (channel>0) then
     Begin
      writeln(TimeStamp+'  Could not detect app ID');
      if logEnabled then writeln(buffer,TimeStamp+'  Could not detect app ID');
     End
    Else if channel=0 then
     Begin
      if not SageIsRecording(tuningDevice,sageIP,sagexUser,sagexPass,buffer,logEnabled) then
       Begin
        KeyPress(rokuIP, 'Home');
        SaveConfig(tuningDevice, channel, appID, homeWaitTime, launchWaitTime,
                   menuDrawWaitTime, menuScrollWaitTime, playWaitTime,
                   sagexUser, sagexPass);
       End;
     End
    Else
     Begin
      Tune(rokuIP,appID,tuningDevice,channel,homeWaitTime,launchWaitTime,
           menuDrawWaitTime,menuScrollWaitTime,playWaitTime,buffer,
           logEnabled);
      SaveConfig(tuningDevice, channel, appID, homeWaitTime, launchWaitTime,
                 menuDrawWaitTime, menuScrollWaitTime, playWaitTime,
                 sagexUser, sagexPass);
     End;
  if logEnabled then closefile(buffer);
 End;


Begin
 Main;
End.
